From mdharm@multivac.one-eyed-alien.net Mon Jun  6 17:24:06 2005
Date: Mon, 6 Jun 2005 17:22:42 -0700
From: Matthew Dharm <mdharm-usb@one-eyed-alien.net>
To: USB Storage List <usb-storage@lists.one-eyed-alien.net>, Greg KH <greg@kroah.com>
Subject: USB Storage: retry hard errors
Message-ID: <20050607002242.GD8689@one-eyed-alien.net>

This patch started life as as527, and was rediffed by me.

Since the IDE interface doesn't convey much information about types of
errors, many USB-IDE adapters report all low-level errors with SK = 0x04,
which is supposed to be used only for non-recoverable errors.  As a result
the SCSI midlayer doesn't retry the command.  But quite often a retry
would succeed, whereas an unnecessary retry doesn't really hurt anything.

This patch uses a recently-implemented flag to tell the SCSI midlayer that 
such hardware errors should be retried.

Signed-off-by: Alan Stern <stern@rowland.harvard.edu>
Signed-off-by: Matthew Dharm <mdharm-usb@one-eyed-alien.net>
Signed-off-by: Greg Kroah-Hartman <gregkh@suse.de>

---
 drivers/usb/storage/scsiglue.c |    9 +++++++++
 1 files changed, 9 insertions(+)

--- gregkh-2.6.orig/drivers/usb/storage/scsiglue.c	2005-06-09 22:47:02.000000000 -0700
+++ gregkh-2.6/drivers/usb/storage/scsiglue.c	2005-06-09 22:47:43.000000000 -0700
@@ -155,6 +155,15 @@
 		 * If this device makes that mistake, tell the sd driver. */
 		if (us->flags & US_FL_FIX_CAPACITY)
 			sdev->fix_capacity = 1;
+
+		/* USB-IDE bridges tend to report SK = 0x04 (Non-recoverable
+		 * Hardware Error) when any low-level error occurs,
+		 * recoverable or not.  Setting this flag tells the SCSI
+		 * midlayer to retry such commands, which frequently will
+		 * succeed and fix the error.  The worst this can lead to
+		 * is an occasional series of retries that will all fail. */
+		sdev->retry_hwerror = 1;
+
 	} else {
 
 		/* Non-disk-type devices don't need to blacklist any pages
